#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Iterator, List, Mapping, Optional

from psym._utils import get_graphql_property_type_inputs
from psym.client import SymphonyClient
from psym.common.cache import PROJECT_TYPES
from psym.common.data_class import (
    ProjectType,
    PropertyDefinition,
    PropertyValue,
    WorkOrderDefinition,
)
from psym.common.data_enum import Entity
from psym.common.data_format import (
    format_to_project_type,
    format_to_property_type_inputs,
)
from psym.exceptions import EntityNotFoundError
from psym.graphql.input.add_project_type_input import AddProjectTypeInput
from psym.graphql.input.work_order_definition_input import WorkOrderDefinitionInput
from psym.graphql.mutation.add_project_type import AddProjectTypeMutation
from psym.graphql.mutation.edit_project_type import (
    EditProjectTypeInput,
    EditProjectTypeMutation,
)
from psym.graphql.mutation.remove_project_type import RemoveProjectTypeMutation
from psym.graphql.query.project_type_details import ProjectTypeDetailsQuery
from psym.graphql.query.project_types import ProjectTypesQuery


def _populate_project_types(client: SymphonyClient) -> None:
    result = ProjectTypesQuery.execute(client)
    if result is None:
        return
    for edge in result.edges:
        node = edge.node
        if node:
            PROJECT_TYPES[node.name] = format_to_project_type(
                project_type_fragment=node
            )


def add_project_type(
    client: SymphonyClient,
    name: str,
    description: Optional[str] = None,
    properties: List[PropertyDefinition] = None,
    work_order_definitions: List[WorkOrderDefinition] = [],
) -> ProjectType:
    """This function creates ProjectType.

    :param name: Project type name
    :type name: str
    :param description: Project type description
    :type description: str, optional
    :param properties: List of property definitions
    :type properties: List[ :class:`~psym.common.data_class.PropertyDefinition` ]
    :param work_order_definitions: List of work order definitions
    :type work_order_definitions: List[ :class:`~psym.common.data_class.WorkOrderDefinition` ]

    :return: Project type
    :rtype: :class:`~psym.common.data_class.ProjectType`

    **Example**

    .. code-block:: python

        client.add_project_type(
            name="Project type",
            description="Project type description",
            properties=[
                PropertyDefinition(
                    property_name="project type property",
                    property_kind=PropertyKind.string,
                    default_raw_value="string value",
                    is_fixed=False,
                )
            ],
            work_order_definitions=[
                WorkOrderDefinition(
                    id="12345678",
                    definition_index="1",
                    work_order_type_id="87654321",
                ),
            ],
        )
    """
    property_type_inputs = []
    if properties is not None:
        property_type_inputs = format_to_property_type_inputs(data=properties)
    result = AddProjectTypeMutation.execute(
        client,
        AddProjectTypeInput(
            name=name,
            description=description,
            properties=property_type_inputs,
            workOrders=[
                WorkOrderDefinitionInput(
                    index=wod.definition_index, type=wod.work_order_type_id
                )
                for wod in work_order_definitions
            ],
        ),
    )
    added = format_to_project_type(project_type_fragment=result)
    PROJECT_TYPES[added.name] = added
    return added


def get_project_types(client: SymphonyClient) -> Iterator[ProjectType]:
    """Get the iterator of project types

    :raises:
        FailedOperationException: Internal symphony error

    :return: ProjectType Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.ProjectType` ]

    **Example**

    .. code-block:: python

        project_types = client.get_project_types()
        for project_type in project_types:
            print(project_type.name, project_type.description)
    """
    result = ProjectTypesQuery.execute(client)
    if result is None:
        return
    for edge in result.edges:
        node = edge.node
        if node is not None:
            yield format_to_project_type(project_type_fragment=node)


def get_project_type_by_id(client: SymphonyClient, id: str) -> ProjectType:
    """This function gets existing ProjectType by its ID.

    :param id: Project type ID
    :type id: str

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Project type does not exist

    :return: Project type
    :rtype: :class:`~psym.common.data_class.ProjectType`

    **Example**

    .. code-block:: python

        client.get_project_type_by_id(id="12345678")
    """
    result = ProjectTypeDetailsQuery.execute(client, id=id)

    if result is None:
        raise EntityNotFoundError(entity=Entity.ProjectType, entity_id=id)

    return format_to_project_type(project_type_fragment=result)


def add_property_types_to_project_type(
    client: SymphonyClient,
    project_type_id: str,
    new_properties: List[PropertyDefinition],
) -> ProjectType:
    """This function adds new property types to existing project type.

    :param project_type_id: Existing project type ID
    :type project_type_id: str
    :param new_properties: List of property definitions
    :type new_properties: List[ :class:`~psym.common.data_class.PropertyDefinition` ]

    :raises:
        FailedOperationException: Internal symphony error

    :return: LocationType object
    :rtype: :class:`~psym.common.data_class.LocationType`

    **Example**

    .. code-block:: python

        project_type = client.add_property_types_to_project_type(
            project_type_id="12345678",
            new_properties=[
                PropertyDefinition(
                    property_name="Contact",
                    property_kind=PropertyKind.string,
                    default_raw_value=None,
                    is_fixed=True
                )
            ],
        )
    """
    project_type = get_project_type_by_id(client=client, id=project_type_id)
    new_property_type_inputs = format_to_property_type_inputs(data=new_properties)
    result = EditProjectTypeMutation.execute(
        client,
        EditProjectTypeInput(
            id=project_type.id,
            name=project_type.name,
            description=project_type.description,
            properties=new_property_type_inputs,
            workOrders=[
                WorkOrderDefinitionInput(
                    index=wod.definition_index, type=wod.work_order_type_id
                )
                for wod in project_type.work_order_definitions
            ],
        ),
    )
    edited = format_to_project_type(project_type_fragment=result)
    PROJECT_TYPES.pop(project_type.name)
    PROJECT_TYPES[edited.name] = edited
    return edited


def edit_project_type(
    client: SymphonyClient,
    project_type_id: str,
    new_name: Optional[str] = None,
    new_description: Optional[str] = None,
    new_properties_defaults: Optional[Mapping[str, PropertyValue]] = None,
    new_work_order_definitions: Optional[List[WorkOrderDefinition]] = [],
) -> ProjectType:
    """This function edits existing ProjectType.

    :param project_type_id: Existing project type ID
    :type project_type_id: str
    :param new_name: Project type new name
    :type new_name: str, optional
    :param new_description: Project type new description
    :type new_description: str, optional
    :param new_properties_defaults: Mapping of property name to property default value

        * str - property name
        * PropertyValue - new default value of the same type for this property

    :type new_properties_defaults: Mapping[str, PropertyValue], optional
    :param new_work_order_definitions: List of new work order definitions
    :type new_work_order_definitions: List[ :class:`~psym.common.data_class.WorkOrderDefinition` ], optional

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Project type does not exist

    :return: Project type
    :rtype: :class:`~psym.common.data_class.ProjectType`

    **Example**

    .. code-block:: python

        edited_project_type = client.edit_project_type(
            project_type_id="12345678,
            new_name="New name",
            new_description="New description",
        )
    """
    project_type = get_project_type_by_id(client=client, id=project_type_id)
    new_name = project_type.name if new_name is None else new_name
    new_description = (
        project_type.description if new_description is None else new_description
    )

    new_property_type_inputs = []
    if new_properties_defaults:
        property_types = project_type.property_types
        new_property_type_inputs = get_graphql_property_type_inputs(
            property_types, new_properties_defaults
        )
    work_order_definitions = (
        project_type.work_order_definitions
        if project_type.work_order_definitions
        else []
    )
    if new_work_order_definitions:
        work_order_definitions = new_work_order_definitions

    result = EditProjectTypeMutation.execute(
        client,
        EditProjectTypeInput(
            id=project_type.id,
            name=new_name,
            description=new_description,
            properties=new_property_type_inputs,
            workOrders=[
                WorkOrderDefinitionInput(
                    index=wod.definition_index, type=wod.work_order_type_id
                )
                for wod in work_order_definitions
            ],
        ),
    )
    edited = format_to_project_type(project_type_fragment=result)
    PROJECT_TYPES.pop(project_type.name)
    PROJECT_TYPES[edited.name] = edited
    return edited


def delete_project_type(client: SymphonyClient, id: str) -> None:
    """This function deletes ProjectType.

    :param id: Project type ID
    :type id: str

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Project type does not exist

    **Example**

    .. code-block:: python

        client.delete_project_type(id="12345678")
    """
    project_type = get_project_type_by_id(client=client, id=id)
    if project_type is None:
        raise EntityNotFoundError(entity=Entity.ProjectType, entity_id=id)
    RemoveProjectTypeMutation.execute(client, id=id)
    del PROJECT_TYPES[project_type.name]



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.distance_unit import DistanceUnit
from ..enum.user_role import UserRole
from ..enum.user_status import UserStatus


@dataclass(frozen=True)
class EditUserInput(DataClassJsonMixin):
    id: str
    distanceUnit: Optional[DistanceUnit] = None
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    organizationFk: Optional[str] = None
    role: Optional[UserRole] = None
    status: Optional[UserStatus] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.




from psym.client import SymphonyClient
from psym.common.data_class import KpiCategory

from ..graphql.input.edit_kpi_category_input import EditKpiCategoryInput
from ..graphql.input.add_kpi_category_input import AddKpiCategoryInput
from ..graphql.mutation.add_kpi_category import AddKpiCategory
from ..graphql.mutation.edit_kpi_category import EditKpiCategory
from ..graphql.input.edit_kpi_category_input import EditKpiCategoryInput
from ..graphql.query.kpi_category import kpiCategoryQuery
from ..graphql.mutation.remove_kpi_category import removeKpiCategory
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional



def add_kpi_category(
    client: SymphonyClient, name: str
) -> KpiCategory:
    """This function adds Kpi Category.

    :param name: Kpi Category name
    :type name: str

    :return: kpiCategory object
    :rtype: :class:`~psym.common.data_class.kpiCategory`

    **Example 1**

    .. code-block:: python

        new_kpi_category = client.add_kpi_category(
            name="kpi_category",
        )
        print(new_kpi_category)
    """
    kpi_category_input = AddKpiCategoryInput(name=name)
    result = AddKpiCategory.execute(client, input=kpi_category_input)
    return KpiCategory(name=result.name, id=result.id)


def edit_kpi_category(
    client: SymphonyClient,
    kpicategory: KpiCategory,
    new_name: Optional[str] = None,
) -> None:
    """This function edits Kpi Category.

    :param kpi_category: Kpi Category entity
    :type name: str
    :param new_name: Kpi Category name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.kpiCategory`

    **Example 1**

    .. code-block:: python

        new_kpi_category_edited = client.edit_kpi_category(
            kpi_category=kpiCategory,
            new_name="kpi_category_edited",

        )
        print(new_kpi_category_edited)

    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        EditKpiCategory.execute(client, input=EditKpiCategoryInput(id=kpicategory.id, name=new_name))


def get_kpi_categories(client: SymphonyClient) -> Iterator[KpiCategory]:
    """ this funtion Get Kpi Categories


    :return: kpiCategory object
    :rtype: Iterator[ :class:`~psym.common.data_class.kpiCategory` ]

    **Example**

    .. code-block:: python

        kpi_categoryes = client.get_kpi_categories()
        for kpi_category in kpi_categories:
            print(kpi_category.name)
    """
    kpi_categories = kpiCategoryQuery.execute(client, first=PAGINATION_STEP)
    edges = kpi_categories.edges if kpi_categories else []
    while kpi_categories is not None and kpi_categories.pageInfo.hasNextPage:
        kpi_categories = kpiCategoryQuery.execute(
            client, after=kpi_categories.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if kpi_categories is not None:
            edges.extend(kpi_categories.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield KpiCategory(
                id=node.id,
                name=node.name,
            )


def delete_kpi_category(client: SymphonyClient, id: str) -> None:
    """This function delete Kpi Category.

    :param name: Kpi Category name
    :type name: :class:`~psym.common.data_class.kpiCategory`
    :rtype: None

    **Example**

    .. code-block:: python

        client.delete_kpi_category(id=123456789)
    """
    removeKpiCategory.execute(client, id=id)






#!/usr/bin/env python3

import re
from typing import Any, Dict, Optional
import requests

from gql import Client
from gql.transport.requests import RequestsHTTPTransport

from requests.auth import HTTPBasicAuth
from requests.models import Response

from .common.endpoint import (
    LOCALHOST_SYMPHONY,
    SYMPHONY_GRAPHQL,
    SYMPHONY_LOGIN,
    SYMPHONY_STORE_DELETE,
    SYMPHONY_STORE_PUT,
    SYMPHONY_URI,
)


class SymphonyClient(Client):
    def __init__(
        self,
        email: str,
        password: str,
        tenant: str,
        app: str = "psym",
        is_local_host: bool = False,
        is_dev_mode: bool = False,
    ) -> None:
        """This is the class to use for working with symphony server.

        The __init__ method uses the credentials to establish session with
        the symphony server. It also consumes graphql schema for
        validations, and validates the client version is compatible with server.

        Args:
            email (str): The email of the user to connect with.
            password (str): The password of the user to connect with.
            tenant (str, optional): The tenant to connect to -
                        should be the beginning of "{}.purpleheadband.cloud"
                        The default is "fb-test" for QA environment
            is_local_host (bool, optional): Used for developers to connect to
                        local symphony. This changes the address and also
                        disable verification of ssl certificate
            is_dev_mode (bool, optional): Used for developers to connect to
                        local symphony from a container. This changes the
                        address and also disable verification of ssl
                        certificate

        """

        self.email = email
        self.password = password
        self.address: str = (
            LOCALHOST_SYMPHONY.format(tenant)
            if is_local_host
            else SYMPHONY_URI.format(tenant)
        )
        graphql_endpoint_address = self.address + SYMPHONY_GRAPHQL
        self.app = app

        auth = HTTPBasicAuth(email, password)
        self.verify_ssl = not is_local_host and not is_dev_mode
        if not self.verify_ssl:
            import urllib3

            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        self.put_endpoint: str = self.address + SYMPHONY_STORE_PUT
        self.delete_endpoint: str = self.address + SYMPHONY_STORE_DELETE

        super().__init__(
            transport=RequestsHTTPTransport(
                url=graphql_endpoint_address,
                headers={
                    "Accept": "application/json",
                    "Content-Type": "application/json",
                    "User-Agent": app,
                },
                auth=auth,
                verify=self.verify_ssl,
            ),
            fetch_schema_from_transport=True,
        )

        self._session: Optional[requests.Session] = None

    def store_file(self, file_path: str, file_type: str, is_global: bool) -> str:
        if self._session is None:
            self._session = self._login()
        sign_response = self._session.get(
            self.put_endpoint,
            params={"contentType": file_type},
            headers={"Is-Global": str(is_global)},
        )
        sign_response_json = sign_response.json()
        signed_url = sign_response_json["URL"]
        with open(file_path, "rb") as f:
            file_data = f.read()
        response = self._session.put(
            signed_url, data=file_data, headers={"Content-Type": file_type}
        )
        response.raise_for_status()
        return sign_response_json["key"]

    def delete_file(self, key: str, is_global: bool) -> None:
        if self._session is None:
            self._session = self._login()
        sign_response = self._session.delete(
            self.delete_endpoint.format(key),
            headers={"Is-Global": str(is_global)},
            allow_redirects=False,
        )
        sign_response.raise_for_status()
        assert sign_response.status_code == 307
        signed_url = sign_response.headers["location"]
        response = self._session.delete(signed_url)
        response.raise_for_status()

    def get(self, url: str) -> Response:
        if self._session is None:
            self._session = self._login()
        return self._session.get(
            "".join([self.address, url]), headers={"User-Agent": self.app}
        )

    def post(self, url: str, json: Optional[Dict[str, Any]] = None) -> Response:
        if self._session is None:
            self._session = self._login()
        return self._session.post(
            "".join([self.address, url]), json=json, headers={"User-Agent": self.app}
        )

    def put(self, url: str, json: Optional[Dict[str, Any]] = None) -> Response:
        if self._session is None:
            self._session = self._login()
        return self._session.put(
            "".join([self.address, url]), json=json, headers={"User-Agent": self.app}
        )

    def _login(self) -> requests.Session:
        session = requests.Session()
        session.verify = self.verify_ssl
        login_endpoint = self.address + SYMPHONY_LOGIN
        response = session.get(login_endpoint)
        match = re.search(b'"csrfToken":"([^"]+)"', response.content)
        assert match is not None, "Problem with symphony login"
        csrf_token = match.group(1).decode("ascii")
        login_data = "_csrf={0}&email={1}&password={2}".format(
            csrf_token, self.email, self.password
        ).encode("ascii")
        response = session.post(
            login_endpoint,
            data=login_data,
            headers={
                "Content-type": "application/x-www-form-urlencoded",
                "User-Agent": self.app,
            },
        )
        response.raise_for_status()
        assert (
            re.search('"email":"{}"'.format(self.email).encode(), response.content)
            is not None
        ), "Credentials are incorrect"
        session.headers.update({"x-csrf-token": csrf_token})
        return session



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class PropertyInput(DataClassJsonMixin):
    propertyTypeID: str
    booleanValue: Optional[bool] = None
    floatValue: Optional[Number] = None
    id: Optional[str] = None
    intValue: Optional[int] = None
    isEditable: Optional[bool] = None
    isInstanceProperty: Optional[bool] = None
    latitudeValue: Optional[Number] = None
    longitudeValue: Optional[Number] = None
    nodeIDValue: Optional[str] = None
    propertyTypeValueID: Optional[str] = None
    rangeFromValue: Optional[Number] = None
    rangeToValue: Optional[Number] = None
    stringValue: Optional[str] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class ActionTypeId(Enum):
    update_inventory = "update_inventory"
    update_workforce = "update_workforce"
    work_order = "work_order"
    worker = "worker"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "ActionTypeId":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class EditRecommendationsInput(DataClassJsonMixin):
    alarmType: str
    externalID: str
    id: str
    longDescription: str
    priority: int
    recommendationsCategory: str
    recommendationsSources: str
    resource: str
    shortDescription: str
    status: bool
    vendor: str
    command: Optional[str] = None
    runbook: Optional[str] = None
    used: Optional[int] = None
    userApprobed: Optional[str] = None



#!/usr/bin/env python3

import copy
import json
import os
from typing import Any, Dict, Tuple

import pkg_resources
from jsonschema import validate
from psym.common.constant import SCHEMA_FILE_NAME


def validate_json(path: str) -> None:
    resource_package = __name__
    schema = pkg_resources.resource_string(resource_package, SCHEMA_FILE_NAME).decode()
    loaded_schema = json.loads(schema)
    with open(path, "rb") as f:
        content = json.loads(f.read().decode())
        validate(content, schema=loaded_schema)


def add_dependencies_to_question(
    question_to_replace: Dict[str, Any], depends_on: Dict[str, Any]
) -> Dict[str, Any]:
    replacable_question = question_to_replace
    while "dependsOn" in replacable_question:
        replacable_question = replacable_question["dependsOn"]
    replacable_question.update({"dependsOn": depends_on})
    return question_to_replace


def set_templates_with_content(
    content: Dict[str, Any],
    template_name_to_questions: Dict[str, Any],
    template_name_to_forms: Dict[str, Any],
) -> None:
    categories = content["categories"]

    changed = True
    while changed:
        changed = False
        for category in categories:
            forms = category["forms"]
            indexes = range(len(forms))
            for i in reversed(indexes):
                if "templateName" in forms[i]:
                    changed = True
                    forms = (
                        forms[:i]
                        + [
                            copy.deepcopy(form)
                            for form in template_name_to_forms[forms[i]["templateName"]]
                        ]
                        + forms[i + 1 :]
                    )
                    category["forms"] = forms
            for form in forms:
                questions = form["questions"]
                indexes = range(len(questions))
                for i in reversed(indexes):
                    if "templateName" in questions[i]:
                        changed = True
                        questions_to_replace = template_name_to_questions[
                            questions[i]["templateName"]
                        ]
                        if "dependsOn" in questions[i]:
                            questions_to_replace = [
                                add_dependencies_to_question(
                                    copy.deepcopy(question_to_replace),
                                    copy.deepcopy(questions[i]["dependsOn"]),
                                )
                                for question_to_replace in questions_to_replace
                            ]
                        questions = (
                            questions[:i] + questions_to_replace + questions[i + 1 :]
                        )
                        form["questions"] = questions


def get_templates_from_content(
    content: Dict[str, Any]
) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    if "templates" not in content:
        return {}, {}
    templates = content["templates"]
    template_name_to_questions = {
        template["templateName"]: template["questions"]
        for template in templates
        if "questions" in template
    }
    template_name_to_forms = {
        template["templateName"]: template["forms"]
        for template in templates
        if "forms" in template
    }
    return template_name_to_questions, template_name_to_forms


def retrieve_tamplates_and_set_them(path: str) -> Dict[str, Any]:
    validate_json(path)
    with open(path, "rb") as f:
        content = json.loads(f.read().decode())
    template_name_to_questions, template_name_to_forms = get_templates_from_content(
        content
    )

    if "imports" in content:
        import_files = content["imports"]
        current_dir_path = os.path.dirname(path)
        for import_file_path in import_files:
            import_file_full_path = os.path.join(current_dir_path, import_file_path)
            validate_json(import_file_full_path)
            import_content = open(import_file_full_path, "rb").read().decode()
            import_content = json.loads(import_content)
            (
                import_question_templates,
                import_form_templates,
            ) = get_templates_from_content(
                import_content  #  type: ignore
            )
            template_name_to_questions.update(import_question_templates)
            template_name_to_forms.update(import_form_templates)
    set_templates_with_content(
        content, template_name_to_questions, template_name_to_forms
    )
    return content


def compile_site_survey_schema(
    input_json_file_path: str, output_json_file_path: str
) -> None:
    content = retrieve_tamplates_and_set_them(input_json_file_path)
    try:
        content.pop("imports")
    except KeyError:
        pass
    try:
        content.pop("templates")
    except KeyError:
        pass
    with open(output_json_file_path, "w") as df:
        json.dump(content, df, indent=4)



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from datetime import datetime
from typing import Dict, Iterator, Optional

from psym._utils import get_graphql_property_inputs
from psym.client import SymphonyClient
from psym.common.cache import WORK_ORDER_TYPES
from psym.common.data_class import PropertyValue, WorkOrder, WorkOrderType, organization
from psym.common.data_enum import Entity
from psym.common.data_format import format_to_work_order
from psym.exceptions import EntityNotFoundError
from psym.graphql.enum.work_order_priority import WorkOrderPriority
from psym.graphql.enum.work_order_status import WorkOrderStatus
from psym.graphql.input.add_work_order_input import AddWorkOrderInput
from psym.graphql.input.edit_work_order_input import EditWorkOrderInput
from psym.graphql.mutation.add_work_order import AddWorkOrderMutation
from psym.graphql.mutation.edit_work_order import EditWorkOrderMutation
from psym.graphql.mutation.remove_work_order import RemoveWorkOrderMutation
from psym.graphql.query.work_order_details import WorkOrderDetailsQuery
from psym.graphql.query.work_orders import WorkOrdersQuery


def add_work_order(
    client: SymphonyClient,
    name: str,
    work_order_type: WorkOrderType,
    organization: str,
    description: Optional[str] = None,
    location_id: Optional[str] = None,
    project_id: Optional[str] = None,
    properties_dict: Dict[str, PropertyValue] = {},
    owner_id: Optional[str] = None,
    assignee_id: Optional[str] = None,
    status: Optional[WorkOrderStatus] = None,
    priority: Optional[WorkOrderPriority] = None,

    
) -> WorkOrder:
    """This function creates work order of with the given name and type

    :param name: Work order name
    :type name: str
    :param description: Work order description
    :type description: str, optional
    :param work_order_type: Work order type object
    :type work_order_type: :class:`~psym.common.data_class.WorkOrderType`
    :param location_id: Existing location ID
    :type location_id: str, optional
    :param project_id: Existing project ID
    :type project_id: str, optional
    :param properties_dict: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Dict[str, PropertyValue]
    :param owner_id: Owner ID
    :type owner_id: str, optional
    :param assignee_id: Assignee ID
    :type assignee_id: str, optional
    :param status: Work order status
    :type status: :class:`~psym.common.data_enum.work_order_status.WorkOrderStatus`, optional
    :param priority: Work order priority
    :type priority: :class:`~psym.common.data_enum.work_order_priority.WorkOrderPriority`, optional

    :return: Work order
    :rtype: :class:`~psym.common.data_class.WorkOrder`

    **Example**

    .. code-block:: python

        work_order_type = client.add_work_order_type(
            name="Work order type",
            description="Work order type description",
            properties=[
                PropertyDefinition(
                    property_name="work order type property",
                    property_kind=PropertyKind.string,
                    default_raw_value="string value",
                    is_fixed=False,
                )
            ],
        )
        client.add_work_order(
            name="new work order",
            work_order_type=work_order_type,
            description="Work order description",
            location_id="12345678",
            project_id="87654321",
            properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            },
            owner_id="81726354",
            assignee_id="18273645",
            status="PLANNED",
            priority="MEDIUM",
        )
    """
    property_types = WORK_ORDER_TYPES[work_order_type.name].property_types
    properties = get_graphql_property_inputs(property_types, properties_dict)
    result = AddWorkOrderMutation.execute(
        client,
        AddWorkOrderInput(
            name=name,
            description=description,
            workOrderTypeId=work_order_type.id,
            locationId=location_id,
            projectId=project_id,
            properties=properties,
            ownerId=owner_id,
            assigneeId=assignee_id,
            status=WorkOrderStatus(status) if status else None,
            priority=WorkOrderPriority(priority) if priority else None,
            checkList=[],
            checkListCategories=[],
            organizationFk=organization,


        ),
    )
    return format_to_work_order(work_order_fragment=result), 


def get_work_orders(client: SymphonyClient) -> Iterator[WorkOrder]:
    """Get the iterator of work orders

    :raises:
        FailedOperationException: Internal symphony error

    :return: WorkOrder Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.WorkOrder` ]

    **Example**

    .. code-block:: python

        work_orders = client.get_work_orders()
        for work_order in work_orders:
            print(work_order.name, work_order.description)
    """
    result = WorkOrdersQuery.execute(client)
    if result is None:
        return
    for edge in result.edges:
        node = edge.node
        if node is not None:
            yield format_to_work_order(work_order_fragment=node)


def get_work_order_by_id(client: SymphonyClient, id: str) -> WorkOrder:
    """Get work order by ID

    :param id: Work order ID
    :type id: str

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Work order does not exist

    :return: WorkOrder
    :rtype: :class:`~psym.common.data_class.WorkOrder`

    **Example**

    .. code-block:: python

        work_order = client.get_work_order_by_id(
            id="12345678",
        )
    """
    result = WorkOrderDetailsQuery.execute(client, id=id)

    if result is None:
        raise EntityNotFoundError(entity=Entity.WorkOrder, entity_id=id)

    return format_to_work_order(work_order_fragment=result)

def get_work_order_by_name(client: SymphonyClient, name: str) -> WorkOrder:
    """This function gets existing WorkOrder by its name.

    :param name: Work order name
    :type name: str

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Work order type does not exist

    :return: Work order type
    :rtype: :class:`~psym.common.data_class.WorkOrder`

    **Example**

    .. code-block:: python

        client.get_work_order_type_by_name(name="WorkOrder Name")
    """

    work_orders = get_work_orders(client=client)
    for work_order in work_orders:
        if work_order.name == name:
            return work_order
    raise EntityNotFoundError(entity=Entity.WorkOrderType, entity_name=name)

def edit_work_order(
    client: SymphonyClient,
    work_order_id: str,
    new_name: Optional[str] = None,
    new_description: Optional[str] = None,
    new_location_id: Optional[str] = None,
    new_project_id: Optional[str] = None,
    new_properties_dict: Optional[Dict[str, PropertyValue]] = None,
    new_owner_id: Optional[str] = None,
    new_assignee_id: Optional[str] = None,
    new_status: Optional[WorkOrderStatus] = None,
    new_priority: Optional[WorkOrderPriority] = None,
    new_organization: Optional[str] = None,
) -> WorkOrder:
    """This function edits existing work order.

    :param work_order_id: Existing work order ID
    :type work_order_id: str
    :param new_name: Work order new name
    :type new_name: str, optional
    :param new_description: Work order new description
    :type new_description: str, optional
    :param new_location_id: Work order new existing location ID
    :type new_location_id: str, optional
    :param new_project_id: Work order new existing project ID
    :type new_project_id: str, optional
    :param new_properties_dict: Dictionary of property name to property default value

        * str - property name
        * PropertyValue - new default value of the same type for this property

    :type new_properties_dict: Dict[str, PropertyValue], optional
    :param new_owner_id: New owner ID
    :type new_owner_id: str, optional
    :param new_assignee_id: New assignee ID
    :type new_assignee_id: str, optional
    :param new_status: New work order status
    :type new_status: :class:`~psym.common.data_enum.work_order_status.WorkOrderStatus`, optional
    :param new_priority: New work order priority
    :type new_priority: :class:`~psym.common.data_enum.work_order_priority.WorkOrderPriority`, optional

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Work order does not exist

    :return: Work order
    :rtype: :class:`~psym.common.data_class.WorkOrder`

    **Example**

    .. code-block:: python

        edited_work_order = client.edit_work_order(
            work_order_id="12345678,
            new_name="New name",
            new_description="New description",
            new_location_id="12345678",
            new_project_id="87654321",
            new_properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            },
            assignee_id="81726354",
            priority="MEDIUM",
        )
    """
    work_order = get_work_order_by_id(client=client, id=work_order_id)
    new_name = work_order.name if new_name is None else new_name
    new_description = (
        work_order.description if new_description is None else new_description
    )
    new_organization = (
        work_order.organization if new_organization is None else new_organization
    )
    new_properties = []
    if new_properties_dict:
        property_types = WORK_ORDER_TYPES[
            work_order.work_order_type_name
        ].property_types
        new_properties = get_graphql_property_inputs(
            property_types, new_properties_dict
        )
    edit_work_order_input = EditWorkOrderInput(
        id=work_order_id,
        name=new_name,
        description=new_description,
        locationId=new_location_id,
        projectId=new_project_id,
        properties=new_properties,
        ownerId=new_owner_id,
        assigneeId=new_assignee_id,
        status=new_status,
        priority=new_priority,
        checkList=[],
        checkListCategories=[],
        organizationFk=new_organization,
        

    )
    result = EditWorkOrderMutation.execute(client, edit_work_order_input)
    return format_to_work_order(work_order_fragment=result)


def delete_work_order(client: SymphonyClient, id: str) -> None:
    """This function deletes WorkOrder.

    :param id: Work order ID
    :type id: str

    :raises:
        FailedOperationException: Internal symphony error

    **Example**

    .. code-block:: python

        client.delete_work_order(id="12345678")
    """
    RemoveWorkOrderMutation.execute(client, id=id)



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.vendor_filter_input import VendorFilterInput
from ..input.vendor_order import VendorOrder


# fmt: off
QUERY: List[str] = ["""
query vendors(
$after: Cursor
$first: Int
$before: Cursor
$last: Int
$orderBy: VendorOrder
$filterBy: [VendorFilterInput!]
){
vendors(after: $after, first: $first, before: $before, last: $last, orderBy: $orderBy, filterBy: $filterBy){
totalCount
edges{
node{
id
name

}
      cursor
    }
    pageInfo{
      hasNextPage
      hasPreviousPage
      startCursor
      endCursor
    }
  }
}
"""
]


class vendors:
    @dataclass(frozen=True)
    class vendorsData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class VendorConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class VendorEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Vendor(DataClassJsonMixin):
                    id: str
                    name: str

                node: Optional[Vendor]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]
                endCursor: Optional[str]

            totalCount: int
            edges: List[VendorEdge]
            pageInfo: PageInfo

        vendors: VendorConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[VendorOrder] = None, filterBy: List[VendorFilterInput] = []) -> vendorsData.VendorConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.vendorsData.from_dict(response_text)
        return res.vendors

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[VendorOrder] = None, filterBy: List[VendorFilterInput] = []) -> vendorsData.VendorConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.vendorsData.from_dict(response_text)
        return res.vendors



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Dict, Iterator, Optional

from psym._utils import get_graphql_property_inputs
from psym.client import SymphonyClient
from psym.common.cache import PROJECT_TYPES
from psym.common.data_class import Project, ProjectType, PropertyValue
from psym.common.data_enum import Entity
from psym.common.data_format import format_to_project
from psym.exceptions import EntityNotFoundError
from psym.graphql.enum.project_priority import ProjectPriority
from psym.graphql.input.add_project_input import AddProjectInput
from psym.graphql.input.edit_project_input import EditProjectInput
from psym.graphql.mutation.add_project import AddProjectMutation
from psym.graphql.mutation.edit_project import EditProjectMutation
from psym.graphql.mutation.remove_project import RemoveProjectMutation
from psym.graphql.query.project_details import ProjectDetailsQuery
from psym.graphql.query.projects import ProjectsQuery


def add_project(
    client: SymphonyClient,
    name: str,
    project_type: ProjectType,
    description: Optional[str] = None,
    priority: Optional[ProjectPriority] = None,
    creator_id: Optional[str] = None,
    location_id: Optional[str] = None,
    properties_dict: Dict[str, PropertyValue] = {},
) -> Project:
    """This function creates Project.

    :param name: Project name
    :type name: str
    :param project_type: Project type object
    :type project_type: :class:`~psym.common.data_class.ProjectType`
    :param description: Project description
    :type description: str, optional
    :param priority: Project priority
    :type priority: :class:`~psym.common.data_enum.project_priority.ProjectPriority`, optional
    :param creator_id: Creator ID
    :type creator_id: str, optional
    :param location_id: Existing location ID
    :type location_id: str, optional
    :param properties_dict: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Dict[str, PropertyValue]

    :return: Project
    :rtype: :class:`~psym.common.data_class.Project`

    **Example**

    .. code-block:: python

        project_type = client.add_project_type(
            name="Project type",
            description="Project type description",
            properties=[
                PropertyDefinition(
                    property_name="project type property",
                    property_kind=PropertyKind.string,
                    default_raw_value="string value",
                    is_fixed=False,
                )
            ],
            work_order_definitions=[
                WorkOrderDefinition(
                    id="12345678",
                    definition_index="1",
                    work_order_type_id="87654321",
                ),
            ],
        )
        client.add_project(
            name="new project",
            project_type=project_type,
            description="Project description",
            priority="MEDIUM",
            creator_id="81726354",
            location_id="12345678",
            properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            },
        )
    """
    property_types = PROJECT_TYPES[project_type.name].property_types
    properties = get_graphql_property_inputs(property_types, properties_dict)
    result = AddProjectMutation.execute(
        client,
        AddProjectInput(
            name=name,
            description=description,
            priority=priority,
            creatorId=creator_id,
            type=project_type.id,
            location=location_id,
            properties=properties,
        ),
    )
    return format_to_project(project_fragment=result)


def edit_project(
    client: SymphonyClient,
    project_id: str,
    new_name: Optional[str] = None,
    new_description: Optional[str] = None,
    new_priority: Optional[ProjectPriority] = None,
    new_creator_id: Optional[str] = None,
    new_location_id: Optional[str] = None,
    new_properties_dict: Optional[Dict[str, PropertyValue]] = None,
) -> Project:
    """This function edits existing project.

    :param project_id: Existing project ID
    :type project_id: str
    :param new_name: Project new name
    :type new_name: str, optional
    :param new_description: Project new description
    :type new_description: str, optional
    :param new_priority: New project priority
    :type new_priority: :class:`~psym.common.data_enum.project_priority.ProjectPriority`, optional
    :param new_creator_id: New creator ID
    :type new_creator_id: str, optional
    :param new_location_id: Project new existing location ID
    :type new_location_id: str, optional
    :param new_properties_dict: Dictionary of property name to property default value

        * str - property name
        * PropertyValue - new default value of the same type for this property

    :type new_properties_dict: Dict[str, PropertyValue], optional

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Project does not exist

    :return: Project
    :rtype: :class:`~psym.common.data_class.Project`

    **Example**

    .. code-block:: python

        edited_project = client.edit_project(
            project_id="12345678,
            new_name="New name",
            new_description="New description",
            new_properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            },
            new_work_order_definitions=[
                WorkOrderDefinition(
                    id="12345678",
                    definition_index="1",
                    work_order_type_id="87654321",
                ),
            ],
        )
    """
    project = get_project_by_id(client=client, id=project_id)
    new_name = project.name if new_name is None else new_name
    new_description = (
        project.description if new_description is None else new_description
    )
    new_properties = []
    if new_properties_dict:
        property_types = PROJECT_TYPES[project.project_type_name].property_types
        new_properties = get_graphql_property_inputs(
            property_types, new_properties_dict
        )
    edit_project_input = EditProjectInput(
        id=project_id,
        name=new_name,
        description=new_description,
        priority=new_priority,
        creatorId=new_creator_id,
        type=project.project_type_id,
        location=new_location_id,
        properties=new_properties,
    )
    result = EditProjectMutation.execute(client, edit_project_input)
    return format_to_project(project_fragment=result)


def get_projects(client: SymphonyClient) -> Iterator[Project]:
    """Get the iterator of projects

    :raises:
        FailedOperationException: Internal symphony error

    :return: Project Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.Project` ]

    **Example**

    .. code-block:: python

        projects = client.get_projects()
        for project in projects:
            print(project.name, project.description)
    """
    result = ProjectsQuery.execute(client)
    if result is None:
        return
    for edge in result.edges:
        node = edge.node
        if node is not None:
            yield format_to_project(project_fragment=node)


def get_project_by_id(client: SymphonyClient, id: str) -> Project:
    """Get project by ID

    :param id: Project ID
    :type id: str

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Project does not exist

    :return: Project
    :rtype: :class:`~psym.common.data_class.Project`

    **Example**

    .. code-block:: python

        project = client.get_project_by_id(
            id="12345678",
        )
    """
    result = ProjectDetailsQuery.execute(client, id=id)

    if result is None:
        raise EntityNotFoundError(entity=Entity.Project, entity_id=id)

    return format_to_project(project_fragment=result)


def delete_project(client: SymphonyClient, id: str) -> None:
    """This function deletes Project.

    :param id: Project ID
    :type id: str

    :raises:
        FailedOperationException: Internal symphony error

    **Example**

    .. code-block:: python

        client.delete_project(id="12345678")
    """
    RemoveProjectMutation.execute(client, id=id)



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from .service_endpoint_definition import ServiceEndpointDefinitionFragment, QUERY as ServiceEndpointDefinitionFragmentQuery

# fmt: off
QUERY: List[str] = ServiceEndpointDefinitionFragmentQuery + ["""
fragment ServiceEndpointFragment on ServiceEndpoint {
  id
  definition {
    ...ServiceEndpointDefinitionFragment
  }
  equipment {
    id
  }
}

"""]

@dataclass(frozen=True)
class ServiceEndpointFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class ServiceEndpointDefinition(ServiceEndpointDefinitionFragment):
        pass

    @dataclass(frozen=True)
    class Equipment(DataClassJsonMixin):
        id: str

    id: str
    definition: ServiceEndpointDefinition
    equipment: Equipment



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import CounterFamily, Vendor, counter

from ..graphql.input.edit_counter_input import EditCounterInput
from ..graphql.input.add_counter_input import AddCounterInput
from ..graphql.mutation.add_counter import addCounter
from ..graphql.mutation.edit_counter import editCounter
from ..graphql.mutation.remove_counter import removeCounter
from ..graphql.query.counters import counters
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_counter(
    client: SymphonyClient, name: str, externalID: str, networkManagerSystem: str,  counterFamily: str, vendor: str
) -> counter:
    """This function adds Counter.

    :param name: Counter name
    :type name: str
    :param id: ID
    :type id: str
    :param externalID: externalID
    :type externalID: str
    :param networkManagerSystem: networkManagerSystem
    :type networkManagerSystem: str
    :param counterFamily: domain Object
    :param vendor: str
    :type counterFamily: psym.common.data_class.counter_family
    :param vendor: str
    :type vendor: psym.common.data_class.vendor

    :return: counter object
    :rtype: :class:`~psym.common.data_class.counter `

    **Example 1**

    .. code-block:: python

        counter_family = client.add_counter_family(
            name="counter_family",
        )

        vendor = client.add_vendor(
            name="vendor",
        )

        counter = client.add_counter(
            name="counter",
            externalID="new counter",
            networkManagerSystem="counter",
            counterFamily=counter_family.id,
            vendor= vendor.id
        )
        print(counter)
    """
    domain_input = AddCounterInput(name=name, 
    externalID=externalID,
    networkManagerSystem=networkManagerSystem, 
    counterFamily=counterFamily,
    vendorFk=vendor)
    result = addCounter.execute(client, input=domain_input)
    return counter(name=result.name, id=result.id, 
    externalID=result.externalID, 
    networkManagerSystem=result.networkManagerSystem, 
    counterFamilyFk=result.counterFamily, 
    vendorFk=result.vendorFk)

def edit_counter(
    client: SymphonyClient,
    counter: counter,
    new_name: Optional[str] = None,
    externalID: str = None,
    networkManagerSystem: str = None,
    vendor: Vendor = None,
) -> None:
    """This function edits Counter.

    :param Counter: Counter entity
    :type name: str
    :param new_name: Alarm Status name
    :type name: str
    :param externalID: externalID
    :type externalID: str
    :param networkManagerSystem: networkManagerSystem
    :type networkManagerSystem: str
    :param counterFamily: domain Object
    :type counterFamily: psym.common.data_class.counter_family
    :param vendor: str
    :type vendor: psym.common.data_class.vendor

    :return: none object
    :rtype: :class:`~psym.common.data_class.alarmStatus`

    **Example 1**

    .. code-block:: python

        u = counter
        Counter_edited=client.edit_counter(
            counter=u,
            new_name=new_name,
            externalID="new counter",
            networkManagerSystem="counter",
            vendor= vendor.id
        )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editCounter.execute(client, input=EditCounterInput(
        id=counter.id, 
        name=new_name,
        externalID=externalID,
        networkManagerSystem=networkManagerSystem,
        vendorFk=vendor,
        ))

def get_counters(client: SymphonyClient) -> Iterator[counter]:
    """ this funtion Get Alarm statuses

    :return: counter object
    :rtype: Iterator[ :class:`~psym.common.data_class.counter` ]

    **Example**

    .. code-block:: python


        counters = client.get_counters()
        for counters in counters:
            print(counter.name)
    """
    counters_ = counters.execute(client, first=PAGINATION_STEP)
    edges = counters_.edges if counters_ else []
    while counters_ is not None and counters_.pageInfo.hasNextPage:
        counters_ = counters.execute(
            client, after=counters_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if counters_ is not None:
            edges.extend(counters_.edges)
    for edge in edges:
        node = edge.node
        if node is not None:
            yield counter(
                id=node.id,
                name=node.name,
                externalID=node.externalID,
                networkManagerSystem=node.networkManagerSystem,
                counterFamilyFk=node.counterFamily,
                vendorFk=node.vendorFk
            )


def remove_counter(client: SymphonyClient, id: str) -> None:
    """This function delete Counter.

    :param name: Counter name
    :type name: :class:`~psym.common.data_class.counter`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_counter(id=123456789)
    """
    removeCounter.execute(client, id=id)












#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class AddRuleLimitInput(DataClassJsonMixin):
    comparator: str
    limitType: str
    number: int
    rule: str



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class FlowNewInstancesPolicy(Enum):
    DISABLED = "DISABLED"
    ENABLED = "ENABLED"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "FlowNewInstancesPolicy":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import rule, threshold, eventSeverity, ruleType

#from ..graphql.input.edit_rule_input import EditRuleInput 
from ..graphql.input.add_rule_input import AddRuleInput
from ..graphql.mutation.add_rule import addRule
#from ..graphql.mutation.edit_rule import editRule
#from ..graphql.mutation.remove_rule import removerule
#from ..graphql.query.rules import rules
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_rule(
    client: SymphonyClient, 
    name: str,
    gracePeriod: int,
    ruleType: str,
    eventTypeName: str,
    specificProblem: str,
    additionalInfo: str,
    status: bool,
    eventSeverity: str,
    threshold: str
) -> rule:
    """This function adds Rule.

    :param name: name
    :type name: str
    :param gracePeriod: gracePeriod
    :type gracePeriod: str
    :param eventTypeName: eventTypeName
    :type eventTypeName: str
    :param specificProblem: specificProblem
    :type specificProblem: str
    :param additionalInfo: additionalInfo
    :type additionalInfo: str
    :param status: status
    :type status: bool
    :param ruleType: str
    :type ruleType: psym.common.data_class.rule_type
    :param eventSeverity: str
    :type eventSeverity: psym.common.data_class.event_severity
    :param threshold: str
    :type threshold: psym.common.data_class.threshold

    :return: rule object
    :rtype: :class:`~psym.common.data_class.rule`

    **Example 1**

    .. code-block:: python

        domain = client.add_domain(
            name="domain",
        )
        kpi_category = client.add_kpi_category(
            name="kpi_category",
        )

        Kpi = client.add_Kpi(
            name="Kpi_2",
            description="new kpi",
            status=True,
            domain=domain.id,
            kpiCategory=kpi_category.id
        )

        threshold = client.add_Threshold(
            name="Threshold",
            description="threshold",
            status=True,
            kpi=Kpi.id
        )
        rule_type = client.add_rule_type(
            name="rule_type",
        )
        event_severity = client.add_event_severity(
            name="event_severity",
        )
        new_rules = client.add_rule(
            name="rule_1",
            additionalInfo="none",
            eventTypeName="none",
            gracePeriod=2,
            specificProblem="none",
            status=True,
            threshold=threshold.id,
            ruleType=rule_type.id,
            eventSeverity=event_severity.id,
            )

    """
    rule_input = AddRuleInput(
    name=name,
    gracePeriod=gracePeriod,
    ruleType=ruleType,
    eventTypeName=eventTypeName,
    specificProblem=specificProblem,
    additionalInfo=additionalInfo,
    status=status,
    eventSeverity=eventSeverity,
    threshold=threshold
    )
    result = addRule.execute(client, input=rule_input)
    return rule(
    name=result.name, 
    id=result.id, 
    gracePeriod=result.gracePeriod,
    ruleType=result.ruleType,
    eventTypeName=result.eventTypeName,
    specificProblem=result.specificProblem,
    additionalInfo=result.additionalInfo,
    status=result.status,
    eventSeverity=result.eventSeverity,
    threshold=result.threshold
    )



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.property_input import PropertyInput


@dataclass(frozen=True)
class AddEquipmentInput(DataClassJsonMixin):
    name: str
    properties: List[PropertyInput]
    type: str
    externalId: Optional[str] = None
    location: Optional[str] = None
    parent: Optional[str] = None
    positionDefinition: Optional[str] = None
    workOrder: Optional[str] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class LifeCycleStatus(Enum):
    INSTALLING = "INSTALLING"
    OPERATING = "OPERATING"
    PLANNING = "PLANNING"
    RETIRED = "RETIRED"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "LifeCycleStatus":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.threshold_filter_input import ThresholdFilterInput
from ..input.threshold_order import ThresholdOrder


# fmt: off
QUERY: List[str] = ["""
query thresholds(
$after: Cursor
$first: Int
$before: Cursor
$last: Int
$orderBy: ThresholdOrder
$filterBy: [ThresholdFilterInput!]
){
thresholds(after: $after, first: $first, before: $before, last: $last, orderBy: $orderBy, filterBy: $filterBy){
totalCount
edges{
node{
id
name
description
status
kpi{id}

}
      cursor
    }
    pageInfo{
      hasNextPage
      hasPreviousPage
      startCursor
      endCursor
    }
  }
}
"""
]


class thresholds:
    @dataclass(frozen=True)
    class thresholdsData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class ThresholdConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class ThresholdEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Threshold(DataClassJsonMixin):
                    @dataclass(frozen=True)
                    class Kpi(DataClassJsonMixin):
                        id: str

                    id: str
                    name: str
                    description: str
                    status: bool
                    kpi: Optional[Kpi]

                node: Optional[Threshold]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]
                endCursor: Optional[str]

            totalCount: int
            edges: List[ThresholdEdge]
            pageInfo: PageInfo

        thresholds: ThresholdConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[ThresholdOrder] = None, filterBy: List[ThresholdFilterInput] = []) -> thresholdsData.ThresholdConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.thresholdsData.from_dict(response_text)
        return res.thresholds

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[ThresholdOrder] = None, filterBy: List[ThresholdFilterInput] = []) -> thresholdsData.ThresholdConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.thresholdsData.from_dict(response_text)
        return res.thresholds



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.service_status import ServiceStatus

from ..input.property_input import PropertyInput


@dataclass(frozen=True)
class ServiceCreateData(DataClassJsonMixin):
    name: str
    serviceTypeId: str
    status: ServiceStatus = _field(metadata=enum_field_metadata(ServiceStatus))
    upstreamServiceIds: List[str]
    customerId: Optional[str] = None
    externalId: Optional[str] = None
    properties: Optional[List[PropertyInput]] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from .property import PropertyFragment, QUERY as PropertyFragmentQuery

from .work_order import WorkOrderFragment, QUERY as WorkOrderFragmentQuery

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.project_priority import ProjectPriority

# fmt: off
QUERY: List[str] = PropertyFragmentQuery + WorkOrderFragmentQuery + ["""
fragment ProjectFragment on Project {
    id
    name
    description
    priority
    createdBy{
        id
    }
    type{
        id
        name
    }
    location{
        id
    }
    workOrders {
        ...WorkOrderFragment
    }
    properties{
        ...PropertyFragment
    }
}
"""]

@dataclass(frozen=True)
class ProjectFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class User(DataClassJsonMixin):
        id: str

    @dataclass(frozen=True)
    class ProjectType(DataClassJsonMixin):
        id: str
        name: str

    @dataclass(frozen=True)
    class Location(DataClassJsonMixin):
        id: str

    @dataclass(frozen=True)
    class WorkOrder(WorkOrderFragment):
        pass

    @dataclass(frozen=True)
    class Property(PropertyFragment):
        pass

    id: str
    name: str
    description: Optional[str]
    priority: ProjectPriority = _field(metadata=enum_field_metadata(ProjectPriority))
    createdBy: Optional[User]
    type: ProjectType
    location: Optional[Location]
    workOrders: List[WorkOrder]
    properties: List[Property]



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.file_type import FileType


@dataclass(frozen=True)
class FileInput(DataClassJsonMixin):
    fileName: str
    storeKey: str
    annotation: Optional[str] = None
    fileType: Optional[FileType] = None
    id: Optional[str] = None
    mimeType: Optional[str] = None
    modificationTime: Optional[int] = None
    sizeInBytes: Optional[int] = None
    uploadTime: Optional[int] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from psym.client import SymphonyClient
from psym.common.data_class import Service
from psym.common.constant import PAGINATION_STEP
from typing import Iterator
from ..graphql.query.services import Services


def get_services(client: SymphonyClient) -> Iterator[Service]:
    """ this funtion Get all Services
    :return: alarmStatus object
    :rtype: Iterator[ :class:`~psym.common.data_class.Service` ]

    **Example**

    .. code-block:: python

        services = client.get_get_services()
        for services_ in services:
            print(services_.name)
    """
    Services_ = Services.execute(client, first=PAGINATION_STEP)
    edges = Services_.edges if Services_ else []
    while Services_ is not None and Services_.pageInfo.hasNextPage:
        Services_ = Services.execute(
            client, after=Services_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if Services_ is not None:
            edges.extend(Services_.edges)
    for edge in edges:
        node = edge.node
        if node is not None:
            yield Service(
                id=node.id,
                name=node.name,
                status=node.status,
                service_type_name=node.serviceType.name
            )



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from .property_type import PropertyTypeFragment, QUERY as PropertyTypeFragmentQuery

# fmt: off
QUERY: List[str] = PropertyTypeFragmentQuery + ["""
fragment ProjectTypeFragment on ProjectType {
    id
    name
    description
    properties {
      ...PropertyTypeFragment
    }
    workOrders {
        id
        index
        type {
            id
            name
        }
    }
}
"""]

@dataclass(frozen=True)
class ProjectTypeFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class PropertyType(PropertyTypeFragment):
        pass

    @dataclass(frozen=True)
    class WorkOrderDefinition(DataClassJsonMixin):
        @dataclass(frozen=True)
        class WorkOrderType(DataClassJsonMixin):
            id: str
            name: str

        id: str
        index: Optional[int]
        type: WorkOrderType

    id: str
    name: str
    description: Optional[str]
    properties: List[PropertyType]
    workOrders: List[WorkOrderDefinition]



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Iterator, Tuple

from psym.client import SymphonyClient
from psym.common.constant import PAGINATION_STEP
from psym.common.data_class import Equipment, Link
from psym.common.data_enum import Entity

from ..exceptions import (
    EntityNotFoundError,
    LinkNotFoundException,
    PortAlreadyOccupiedException,
)
from ..graphql.input.add_link_input import AddLinkInput
from ..graphql.input.link_side import LinkSide
from ..graphql.mutation.add_link import AddLinkMutation
from ..graphql.query.equipment_ports import EquipmentPortsQuery
from ..graphql.query.links import LinksQuery
from .port import get_port


def get_all_links_and_port_names_of_equipment(
    client: SymphonyClient, equipment: Equipment
) -> Iterator[Tuple[Link, str]]:
    """Returns all links and port names in equipment.

    :param equipment: Equipment object to be copied, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`

    :raises:
        * `psym.exceptions.EntityNotFoundError`: Link not found
        * FailedOperationException: Internal symphony error

    :return: Iterator of Tuple[Link, str]

        * Link - :class:`~psym.common.data_class.Link`
        * str - port name

    :rtype: Iterator[ Tuple[ :class:`~psym.common.data_class.Link`, str ] ]

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location1)
        client.get_all_links_and_port_names_of_equipment(equipment=equipment)
    """

    equipment_data = EquipmentPortsQuery.execute(client, id=equipment.id)
    if equipment_data is None:
        raise EntityNotFoundError(entity=Entity.Equipment, entity_id=equipment.id)
    ports = equipment_data.ports
    for port in ports:
        port_link = port.link
        if port_link is not None:
            yield (
                Link(
                    id=port_link.id,
                    properties=port_link.properties,
                    service_ids=[s.id for s in port_link.services if port_link.services]
                    if port_link.services is not None
                    else [],
                ),
                port.definition.name,
            )


def add_link(
    client: SymphonyClient,
    equipment_a: Equipment,
    port_name_a: str,
    equipment_b: Equipment,
    port_name_b: str,
) -> Link:
    """Connects a link between two ports of two equipments.

    :param equipment_a: Equipment object to connect, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment_a: :class:`~psym.common.data_class.Equipment`
    :param port_name_a: Name of the port in equipment A
    :type port_name_a: str
    :param equipment_b: Equipment object to connect, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment_b: :class:`~psym.common.data_class.Equipment`
    :param port_name_b: Name of the port in equipment B
    :type port_name_b: str

    :raises:
        * AssertionError: `port_name` in any of the equipments does not exist,
          or match more than one port, or is already occupied by link
        * FailedOperationException: Internal symphony error

    :return: Link object
    :rtype: :class:`~psym.common.data_class.Link`

    **Example**

    .. code-block:: python

        location1 = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment1 = client.get_equipment(name="indProdCpy1_AIO", location=location1)
        location2 = client.get_location({("Country", "LS_IND_Prod")})
        equipment2 = client.get_equipment(name="indProd1_AIO", location=location2)
        client.add_link(
            equipment_a=equipment1,
            port_name_a="Port A",
            equipment_b=equipment2,
            port_name_b="Port B"
        )
    """

    port_a = get_port(client, equipment_a, port_name_a)
    if port_a.link is not None:
        raise PortAlreadyOccupiedException(equipment_a.name, port_a.definition.name)
    port_a_definition_id = port_a.definition.id
    if not port_a_definition_id:
        raise EntityNotFoundError(
            entity=Entity.EquipmentPort, msg="port definition has no ID"
        )
    port_b = get_port(client, equipment_b, port_name_b)
    if port_b.link is not None:
        raise PortAlreadyOccupiedException(equipment_b.name, port_b.definition.name)
    port_b_definition_id = port_b.definition.id
    if not port_b_definition_id:
        raise EntityNotFoundError(
            entity=Entity.EquipmentPort, msg="port definition has no ID"
        )

    add_link_input = AddLinkInput(
        sides=[
            LinkSide(equipment=equipment_a.id, port=port_a_definition_id),
            LinkSide(equipment=equipment_b.id, port=port_b_definition_id),
        ],
        properties=[],
        serviceIds=[],
    )
    link = AddLinkMutation.execute(client, add_link_input)

    return Link(
        id=link.id,
        properties=link.properties,
        service_ids=[s.id for s in link.services],
    )


def get_links(client: SymphonyClient) -> Iterator[Link]:
    """This function returns all existing links

    :return: Links Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.Link` ]

    **Example**

    .. code-block:: python

        all_links = client.get_links()
    """
    links = LinksQuery.execute(client, first=PAGINATION_STEP)
    edges = links.edges if links else []
    while links is not None and links.pageInfo.hasNextPage:
        links = LinksQuery.execute(
            client, after=links.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if links is not None:
            edges.extend(links.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield Link(
                id=node.id,
                properties=node.properties,
                service_ids=[s.id for s in node.services],
            )


def get_link_in_port_of_equipment(
    client: SymphonyClient, equipment: Equipment, port_name: str
) -> Link:
    """Returns link in specific port by name in equipment.

    :param equipment: Equipment object that has link, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param port_name: Name of the port in equipment
    :type port_name: str

    :raises:
        * LinkNotFoundException: Link not found
        * FailedOperationException: Internal symphony error

    :return: Link object
    :rtype: :class:`~psym.common.data_class.Link`

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        client.get_link_in_port_of_equipment(
            equipment=equipment,
            port_name="Port A"
        )
    """
    port = get_port(client, equipment, port_name)
    link = port.link
    if link is not None:
        return Link(
            id=link.id, properties=link.properties, service_ids=link.service_ids
        )
    raise LinkNotFoundException(equipment.name, port_name)



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.service_status import ServiceStatus

from ..input.service_filter_input import ServiceFilterInput


# fmt: off
QUERY: List[str] = ["""
query Services($after: Cursor, $before: Cursor, $filterBy: [ServiceFilterInput!], $first: Int, $last: Int) {
  services(after: $after, before: $before, filterBy: $filterBy, first: $first, last: $last) {
    edges {
      node {
        id
        name
        status
        customer {
          name
          id
        }
        serviceType {
          id
          name
        }
        properties {
          id
          stringValue
          rawValue
          rangeToValue
          rangeFromValue

          longitudeValue
          latitudeValue
          intValue
          floatValue
          booleanValue
        }
      }
      cursor
    }
    pageInfo {
      endCursor
      hasNextPage
      hasPreviousPage
      startCursor
    }

  }
}
"""
]


class Services:
    @dataclass(frozen=True)
    class ServicesData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class ServiceConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class ServiceEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Service(DataClassJsonMixin):
                    @dataclass(frozen=True)
                    class Customer(DataClassJsonMixin):
                        name: str
                        id: str

                    @dataclass(frozen=True)
                    class ServiceType(DataClassJsonMixin):
                        id: str
                        name: str

                    @dataclass(frozen=True)
                    class Property(DataClassJsonMixin):
                        id: str
                        stringValue: Optional[str]
                        rawValue: Optional[str]
                        rangeToValue: Optional[Number]
                        rangeFromValue: Optional[Number]
                        longitudeValue: Optional[Number]
                        latitudeValue: Optional[Number]
                        intValue: Optional[int]
                        floatValue: Optional[Number]
                        booleanValue: Optional[bool]

                    id: str
                    name: str
                    status: ServiceStatus = _field(metadata=enum_field_metadata(ServiceStatus))
                    customer: Optional[Customer]
                    serviceType: ServiceType
                    properties: List[Property]

                node: Optional[Service]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                endCursor: Optional[str]
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]

            edges: List[ServiceEdge]
            pageInfo: PageInfo

        services: ServiceConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, before: Optional[str] = None, filterBy: List[ServiceFilterInput] = [], first: Optional[int] = None, last: Optional[int] = None) -> ServicesData.ServiceConnection:
        variables: Dict[str, Any] = {"after": after, "before": before, "filterBy": filterBy, "first": first, "last": last}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.ServicesData.from_dict(response_text)
        return res.services

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, before: Optional[str] = None, filterBy: List[ServiceFilterInput] = [], first: Optional[int] = None, last: Optional[int] = None) -> ServicesData.ServiceConnection:
        variables: Dict[str, Any] = {"after": after, "before": before, "filterBy": filterBy, "first": first, "last": last}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.ServicesData.from_dict(response_text)
        return res.services



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

SYMPHONY_URI = "https://{}"
LOCALHOST_SERVER = "localtest.me"
LOCALHOST_SYMPHONY = f"https://{{}}.{LOCALHOST_SERVER}"
SYMPHONY_GRAPHQL = "/apollo"
SYMPHONY_STORE_PUT = "/store/put"
SYMPHONY_LOGIN = "/user/login"
SYMPHONY_STORE_DELETE = "/store/delete?key={}"



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class WorkOrderStatus(Enum):
    BLOCKED = "BLOCKED"
    CANCELED = "CANCELED"
    CLOSED = "CLOSED"
    DONE = "DONE"
    IN_PROGRESS = "IN_PROGRESS"
    PENDING = "PENDING"
    PLANNED = "PLANNED"
    SUBMITTED = "SUBMITTED"
    SUSPENDED = "SUSPENDED"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "WorkOrderStatus":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.




from psym.client import SymphonyClient
from psym.common.data_class import KqiSource

from ..graphql.input.edit_kqi_source_input import EditKqiSourceInput
from ..graphql.input.add_kqi_source_input import AddKqiSourceInput
from ..graphql.input.edit_kqi_source_input import EditKqiSourceInput
from ..graphql.mutation.add_kqi_source import addKqiSource
from ..graphql.mutation.edit_kqi_source import editKqiSource
from ..graphql.query.kqi_source import kqiSources
from ..graphql.mutation.remove_kqi_source import removeKqiSource
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_kqi_source(
    client: SymphonyClient, name: str
) -> KqiSource:
    """This function adds Kqi Source.

    :param name: Kqi Source name
    :type name: str

    :return: KqiSource object
    :rtype: :class:`~psym.common.data_class.KqiSource`

    **Example 1**

    .. code-block:: python

        new_kqi_source = client.add_kqi_source(
            name="kqi_source",

        )
    """
    kqi_source_input = AddKqiSourceInput(name=name)
    result = addKqiSource.execute(client, input=kqi_source_input)
    return KqiSource(name=result.name, id=result.id)


def edit_kqi_source(
    client: SymphonyClient,
    kqisource: KqiSource,
    new_name: Optional[str] = None,
) -> None:
    """This function edits Kqi Source.

    :param kqi_source: Kqi Source entity
    :type name: str
    :param new_name: Kqi Source name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.KqiSource`

    **Example 1**

    .. code-block:: python

        new_kqi_source_edited = client.edit_kqi_source(
            kqi_source=KqiSource,
            new_name="kqi_source_edited",

        )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editKqiSource.execute(client, input=EditKqiSourceInput(id=kqisource.id, name=new_name))


def get_kqi_sources(client: SymphonyClient) -> Iterator[KqiSource]:
    """ this funtion Get Kqi Sources


    :return: KqiSource object
    :rtype: Iterator[ :class:`~psym.common.data_class.KqiSource` ]

    **Example**

    .. code-block:: python

        kqi_sourcees = client.get_kqi_sources()
        for kqi_source in kqi_sources:
            print(kqi_source.name)
    """
    kqi_sources = kqiSources.execute(client, first=PAGINATION_STEP)
    edges = kqi_sources.edges if kqi_sources else []
    while kqi_sources is not None and kqi_sources.pageInfo.hasNextPage:
        kqi_sources = kqiSources.execute(
            client, after=kqi_sources.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if kqi_sources is not None:
            edges.extend(kqi_sources.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield KqiSource(
                id=node.id,
                name=node.name,
            )


def delete_kqi_source(client: SymphonyClient, id: str) -> None:
    """This function delete Kqi Source.

    :param name: Kqi Source name
    :type name: :class:`~psym.common.data_class.KqiSource`
    :rtype: None

    **Example**

    .. code-block:: python

        client.delete_kqi_source(id=123456789)
    """
    removeKqiSource.execute(client, id=id)






#!/usr/bin/env python3

import codecs
import os
import re

import setuptools


def read(*parts):
    with codecs.open(os.path.join(*parts), "r") as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


PSYM_PACKAGES = ["psym", "psym.*"]

this_directory = os.path.abspath(os.path.dirname(__file__))
long_description = ""
with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()


setuptools.setup(
    name="psym_nttdata",
    version=find_version("psym", "common", "constant.py"),
    author="Facebook Inc.",
    url="https://github.com/facebookincubator/symphony",
    description="Tool for accessing and modifying Symphony database",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(include=PSYM_PACKAGES),
    license="BSD License",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3.7",
    ],
    python_requires=">=3.7",
    include_package_data=True,
    install_requires=[
        "py-gql-client>=1.0.0",
        "tqdm>=4.32.2",
        "unicodecsv>=0.14.1",
        "requests>=2.22.0",
        "requests-toolbelt>=0.9.1",
        "filetype>=1.0.5",
        "jsonschema",
        "pandas>=0.24.2",
        "xlsxwriter>=1.1.8",
        "dacite>=1.0.2",
    ],
)



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.check_list_category_definition_input import CheckListCategoryDefinitionInput
from ..input.property_type_input import PropertyTypeInput


@dataclass(frozen=True)
class EditWorkOrderTypeInput(DataClassJsonMixin):
    checkListCategories: List[CheckListCategoryDefinitionInput]
    id: str
    name: str
    assigneeCanCompleteWorkOrder: Optional[bool] = None
    description: Optional[str] = None
    duration: Optional[Number] = None
    hasCosts: Optional[bool] = None
    hasMaterials: Optional[bool] = None
    properties: Optional[List[PropertyTypeInput]] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.formula_filter_input import FormulaFilterInput
from ..input.formula_order import FormulaOrder


# fmt: off
QUERY: List[str] = ["""
query formulas(
$after: Cursor
$first: Int
$before: Cursor
$last: Int
$orderBy: FormulaOrder
$filterBy: [FormulaFilterInput!]
){
formulas(after: $after, first: $first, before: $before, last: $last, orderBy: $orderBy, filterBy: $filterBy){
totalCount
edges{
node{
id
textFormula
status
networkTypeFk{id}
kpiFk{id}
techFk{id}

}
      cursor
    }
    pageInfo{
      hasNextPage
      hasPreviousPage
      startCursor
      endCursor
    }
  }
}
"""
]


class formulas:
    @dataclass(frozen=True)
    class formulasData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class FormulaConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class FormulaEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Formula(DataClassJsonMixin):
                    @dataclass(frozen=True)
                    class NetworkType(DataClassJsonMixin):
                        id: str

                    @dataclass(frozen=True)
                    class Kpi(DataClassJsonMixin):
                        id: str

                    @dataclass(frozen=True)
                    class Tech(DataClassJsonMixin):
                        id: str

                    id: str
                    textFormula: Optional[str]
                    status: bool
                    networkTypeFk: NetworkType
                    kpiFk: Kpi
                    techFk: Tech

                node: Optional[Formula]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]
                endCursor: Optional[str]

            totalCount: int
            edges: List[FormulaEdge]
            pageInfo: PageInfo

        formulas: FormulaConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[FormulaOrder] = None, filterBy: List[FormulaFilterInput] = []) -> formulasData.FormulaConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.formulasData.from_dict(response_text)
        return res.formulas

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[FormulaOrder] = None, filterBy: List[FormulaFilterInput] = []) -> formulasData.FormulaConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.formulasData.from_dict(response_text)
        return res.formulas
